@@ -24,7 +24,7 @@ Testing:
     xdoctest ubelt
 """
 
-__version__ = '1.2.4'
+__version__ = '1.3.4'
 
 # Deprecated functions
 from ubelt.util_platform import (
@@ -35,6 +35,8 @@ from ubelt.util_platform import (
 
 from ubelt.util_io import (readfrom, writeto,)
 from ubelt.util_str import (ensure_unicode,)
+from ubelt import util_format
+from ubelt.util_format import FormatterExtensions, repr2
 
 
 __ignore__ = [
@@ -59,6 +61,9 @@ __explicit__ = [
     'readfrom',
     'writeto',
     'ensure_unicode',
+    'util_format',
+    'repr2',
+    'FormatterExtensions',
 ]
 
 
@@ -73,7 +78,7 @@ __submodules__ = {
     'util_download': None,
     'util_download_manager': None,
     'util_func': None,
-    'util_format': None,
+    'util_repr': None,
     'util_futures': None,
     'util_io': None,
     'util_links': None,
@@ -104,7 +109,6 @@ from ubelt import util_deprecate
 from ubelt import util_dict
 from ubelt import util_download
 from ubelt import util_download_manager
-from ubelt import util_format
 from ubelt import util_func
 from ubelt import util_futures
 from ubelt import util_hash
@@ -117,6 +121,7 @@ from ubelt import util_memoize
 from ubelt import util_mixins
 from ubelt import util_path
 from ubelt import util_platform
+from ubelt import util_repr
 from ubelt import util_str
 from ubelt import util_stream
 from ubelt import util_time
@@ -137,7 +142,7 @@ from ubelt.util_deprecate import (schedule_deprecation,)
 from ubelt.util_download import (download, grabdata,)
 from ubelt.util_download_manager import (DownloadManager,)
 from ubelt.util_func import (compatible, identity, inject_method,)
-from ubelt.util_format import (FormatterExtensions, repr2, urepr,)
+from ubelt.util_repr import (ReprExtensions, urepr,)
 from ubelt.util_futures import (Executor, JobPool,)
 from ubelt.util_io import (delete, touch,)
 from ubelt.util_links import (symlink,)
@@ -151,8 +156,8 @@ from ubelt.util_import import (import_module_from_name,
 from ubelt.util_indexable import (IndexableWalker, indexable_allclose,)
 from ubelt.util_memoize import (memoize, memoize_method, memoize_property,)
 from ubelt.util_mixins import (NiceRepr,)
-from ubelt.util_path import (Path, TempDir, augpath, ensuredir, expandpath,
-                             shrinkuser, userhome,)
+from ubelt.util_path import (ChDir, Path, TempDir, augpath, ensuredir,
+                             expandpath, shrinkuser, userhome,)
 from ubelt.util_platform import (DARWIN, LINUX, POSIX, WIN32, find_exe,
                                  find_path, platform_cache_dir,
                                  platform_config_dir, platform_data_dir,)
@@ -164,10 +169,11 @@ from ubelt.orderedset import (OrderedSet, oset,)
 from ubelt.progiter import (ProgIter,)
 
 __all__ = ['AutoDict', 'AutoOrderedDict', 'CacheStamp', 'Cacher',
-           'CaptureStdout', 'CaptureStream', 'DARWIN', 'DownloadManager',
-           'Executor', 'FormatterExtensions', 'IndexableWalker', 'JobPool',
-           'LINUX', 'NO_COLOR', 'NiceRepr', 'NoParam', 'OrderedSet', 'POSIX',
-           'Path', 'ProgIter', 'SetDict', 'TeeStringIO', 'TempDir', 'Timer',
+           'CaptureStdout', 'CaptureStream', 'ChDir', 'DARWIN',
+           'DownloadManager', 'Executor', 'FormatterExtensions',
+           'IndexableWalker', 'JobPool', 'LINUX', 'NO_COLOR', 'NiceRepr',
+           'NoParam', 'OrderedSet', 'POSIX', 'Path', 'ProgIter',
+           'ReprExtensions', 'SetDict', 'TeeStringIO', 'TempDir', 'Timer',
            'UDict', 'WIN32', 'allsame', 'argflag', 'argmax', 'argmin',
            'argsort', 'argunique', 'argval', 'augpath', 'boolmask', 'chunks',
            'cmd', 'codeblock', 'color_text', 'compatible', 'compress', 'ddict',
@@ -188,11 +194,12 @@ __all__ = ['AutoDict', 'AutoOrderedDict', 'CacheStamp', 'Cacher',
            'repr2', 'schedule_deprecation', 'sdict', 'shrinkuser',
            'sorted_keys', 'sorted_vals', 'sorted_values', 'split_archive',
            'split_modpath', 'symlink', 'take', 'timeparse', 'timestamp',
-           'touch', 'udict', 'unique', 'unique_flags', 'userhome', 'urepr',
+           'touch', 'udict', 'unique', 'unique_flags', 'urepr', 'userhome',
            'util_arg', 'util_cache', 'util_cmd', 'util_colors', 'util_const',
            'util_deprecate', 'util_dict', 'util_download',
            'util_download_manager', 'util_format', 'util_func', 'util_futures',
            'util_hash', 'util_import', 'util_indexable', 'util_io',
            'util_links', 'util_list', 'util_memoize', 'util_mixins',
-           'util_path', 'util_platform', 'util_str', 'util_stream',
-           'util_time', 'util_zip', 'varied_values', 'writeto', 'zopen']
+           'util_path', 'util_platform', 'util_repr', 'util_str',
+           'util_stream', 'util_time', 'util_zip', 'varied_values', 'writeto',
+           'zopen']
