@@ -56,7 +56,6 @@ import operator as op
 import itertools as it
 from collections import OrderedDict
 from collections import defaultdict
-# from ubelt import util_const
 from ubelt.util_const import NoParam
 
 __all__ = [
@@ -114,7 +113,7 @@ def dzip(items1, items2, cls=dict):
             can either be a sequence of one item or a sequence of equal length
             to ``items1``
 
-        cls (Type[dict], default=dict): dictionary type to use.
+        cls (Type[dict]): dictionary type to use. Defaults to ``dict``.
 
     Returns:
         Dict[KT, VT]: similar to ``dict(zip(items1, items2))``.
@@ -194,16 +193,19 @@ def dict_hist(items, weights=None, ordered=False, labels=None):
         items (Iterable[T]):
             hashable items (usually containing duplicates)
 
-        weights (Iterable[float] | None, default=None):
+        weights (Iterable[float] | None):
             Corresponding weights for each item, defaults to 1 if unspecified.
+            Defaults to None.
 
-        ordered (bool, default=False):
+        ordered (bool):
             If True the result is ordered by frequency.
+            Defaults to False.
 
-        labels (Iterable[T] | None, default=None):
+        labels (Iterable[T] | None):
             Expected labels.  Allows this function to pre-initialize the
             histogram.  If specified the frequency of each label is initialized
             to zero and ``items`` can only contain items specified in labels.
+            Defaults to None.
 
     Returns:
         dict[T, int] :
@@ -273,18 +275,19 @@ def find_duplicates(items, k=2, key=None):
 
     Args:
         items (Iterable[T]):
-            hashable items possibly containing duplicates
+            Hashable items possibly containing duplicates
 
-        k (int, default=2):
-            only return items that appear at least ``k`` times.
+        k (int):
+            Only return items that appear at least ``k`` times.
+            Defaults to 2.
 
-        key (Callable[[T], Any] | None, default=None):
+        key (Callable[[T], Any] | None):
             Returns indices where `key(items[i])` maps to a particular value at
-            least k times.
+            least k times. Default to None.
 
     Returns:
         dict[T, List[int]] :
-            maps each duplicate item to the indices at which it appears
+            Maps each duplicate item to the indices at which it appears
 
     Notes:
         Similar to :func:`more_itertools.duplicates_everseen`,
@@ -344,7 +347,8 @@ def dict_subset(dict_, keys, default=NoParam, cls=OrderedDict):
         default (Optional[object] | NoParamType):
             if specified uses default if keys are missing.
 
-        cls (Type[Dict], default=OrderedDict): type of the returned dictionary.
+        cls (Type[Dict]): type of the returned dictionary.
+            Defaults to ``OrderedDict``.
 
     Returns:
         Dict[KT, VT]: subset dictionary
@@ -402,10 +406,11 @@ def dict_union(*args):
         >>> import ubelt as ub
         >>> result = ub.dict_union({'a': 1, 'b': 1}, {'b': 2, 'c': 2})
         >>> assert result == {'a': 1, 'b': 2, 'c': 2}
-        >>> ub.dict_union(
+        >>> output = ub.dict_union(
         >>>     ub.odict([('a', 1), ('b', 2)]),
         >>>     ub.odict([('c', 3), ('d', 4)]))
-        OrderedDict([('a', 1), ('b', 2), ('c', 3), ('d', 4)])
+        >>> print(ub.urepr(output, nl=0))
+        {'a': 1, 'b': 2, 'c': 3, 'd': 4}
         >>> ub.dict_union()
         {}
     """
@@ -440,8 +445,9 @@ def dict_diff(*args):
         >>> import ubelt as ub
         >>> ub.dict_diff({'a': 1, 'b': 1}, {'a'}, {'c'})
         {'b': 1}
-        >>> ub.dict_diff(odict([('a', 1), ('b', 2)]), odict([('c', 3)]))
-        OrderedDict([('a', 1), ('b', 2)])
+        >>> result = ub.dict_diff(ub.odict([('a', 1), ('b', 2)]), ub.odict([('c', 3)]))
+        >>> print(ub.urepr(result, nl=0))
+        {'a': 1, 'b': 2}
         >>> ub.dict_diff()
         {}
         >>> ub.dict_diff({'a': 1, 'b': 2}, {'c'})
@@ -614,10 +620,10 @@ def sorted_values(dict_, key=None, reverse=False, cls=OrderedDict):
             If given as a callable, customizes the sorting by ordering using
             transformed values.
 
-        reverse (bool, default=False):
-            if True returns in descending order
+        reverse (bool, default):
+            If True returns in descending order. Defaults to False.
 
-        cls (type): specifies the dict return type
+        cls (type): Specifies the dict return type. Default to OrderedDict.
 
     SeeAlso:
         ::py:meth:`UDict.sorted_values` - object oriented version of this function
@@ -656,14 +662,14 @@ def sorted_keys(dict_, key=None, reverse=False, cls=OrderedDict):
 
     Args:
         dict_ (Dict[KT, VT]):
-            dictionary to sort. The keys must be of comparable types.
+            Dictionary to sort. The keys must be of comparable types.
 
         key (Callable[[KT], Any] | None):
             If given as a callable, customizes the sorting by ordering using
             transformed keys.
 
-        reverse (bool, default=False):
-            if True returns in descending order
+        reverse (bool):
+            If True returns in descending order. Default to False.
 
         cls (type): specifies the dict return type
 
@@ -702,8 +708,8 @@ def invert_dict(dict_, unique_vals=True, cls=None):
     Args:
         dict_ (Dict[KT, VT]): dictionary to invert
 
-        unique_vals (bool, default=True): if False, the values of the new
-            dictionary are sets of the original keys.
+        unique_vals (bool): if False, the values of the new
+            dictionary are sets of the original keys. Defaults to True.
 
         cls (type | None): specifies the dict subclassof the result.
             if unspecified will be dict or OrderedDict. This behavior may
@@ -770,9 +776,9 @@ def named_product(_=None, **basis):
     variables to values).
 
     Args:
-        _ (Dict[str, List[VT]] | None, default=None):
+        _ (Dict[str, List[VT]] | None):
             Use of this positional argument is not recommend. Instead specify
-            all arguments as keyword args.
+            all arguments as keyword args. Defaults to None.
 
             If specified, this should be a dictionary is unioned with the
             keyword args.  This exists to support ordered dictionaries before
@@ -873,9 +879,9 @@ def varied_values(longform, min_variations=0, default=NoParam):
             The values of the dictionary must be hashable. Lists will be
             converted into tuples.
 
-        min_variations (int, default=0):
+        min_variations (int):
             "columns" with fewer than ``min_variations`` unique values are
-            removed from the result.
+            removed from the result. Defaults to 0.
 
         default (VT | NoParamType):
             if specified, unspecified columns are given this value.
@@ -1006,13 +1012,122 @@ class SetDict(dict):
     A dictionary subclass where all set operations are defined.
 
     All of the set operations are defined in a key-wise fashion, that is it is
-    like performing the operation on sets of keys.
+    like performing the operation on sets of keys. Value conflicts are handled
+    with left-most priority (default for ``intersection`` and ``difference``),
+    right-most priority (default for ``union`` and ``symmetric_difference``),
+    or via a custom ``merge`` callable similar to [RubyMerge]_.
+
+    The set operations are:
+
+        * union (or the ``|`` operator) combines multiple dictionaries into
+            one. This is nearly identical to the update operation. Rightmost
+            values take priority.
+
+        * intersection (or the ``&`` operator). Takes the items from the
+            first dictionary that share keys with the following dictionaries
+            (or lists or sets of keys).  Leftmost values take priority.
+
+        * difference (or the ``-`` operator). Takes only items from the first
+            dictionary that do not share keys with following dictionaries.
+            Leftmost values take priority.
+
+        * symmetric_difference (or the ``^`` operator). Takes the items
+            from all dictionaries where the key appears an odd number of times.
+            Rightmost values take priority.
+
+    The full set of set operations was originally determined to be beyond the
+    scope of [Pep584]_, but there was discussion of these additional
+    operations. Some choices were ambiguous, but we believe this design could
+    be considered "natural".
+
+    Note:
+        By default the right-most values take priority in union /
+        symmetric_difference and left-most values take priority in intersection
+        / difference. In summary this is because we consider intersection /
+        difference to be "subtractive" operations, and union /
+        symmetric_difference to be "additive" operations. We expand on this in
+        the following points:
+
+            1. intersection / difference is for removing keys --- i.e. is used
+            to find values in the first (main) dictionary that are also in some
+            other dictionary (or set or list of keys), whereas
+
+            2. union is for adding keys --- i.e. it is basically just an alias
+            for dict.update, so the new (rightmost) keys clobber the old.
+
+            3. symmetric_difference is somewhat strange if you aren't familiar
+            with it. At a pure-set level it's not really a difference, its a
+            pairty operation (think of it more like xor or addition modulo 2).
+            You only keep items where the key appears an odd number of times.
+            Unlike intersection and difference, the results may not be a subset
+            of either input. The union has the same property. This symmetry
+            motivates having the newest (rightmost) keys cobber the old.
+
+        Also, union / symmetric_difference does not make sense if arguments on
+        the rights are lists/sets, whereas difference / intersection does.
 
     Note:
         The SetDict class only defines key-wise set operations.  Value-wise or
         item-wise operations are in general not hashable and therefore not
         supported. A heavier extension would be needed for that.
 
+    TODO:
+        - [ ] implement merge callables so the user can specify how to resolve
+              value conflicts / combine values.
+
+    References:
+        .. [RubyMerge] https://ruby-doc.org/core-2.7.0/Hash.html#method-i-merge
+        .. [Pep584] https://peps.python.org/pep-0584/#what-about-the-full-set-api
+
+    CommandLine:
+        xdoctest -m ubelt.util_dict SetDict
+
+    Example:
+        >>> import ubelt as ub
+        >>> a = ub.SetDict({'A': 'Aa', 'B': 'Ba',            'D': 'Da'})
+        >>> b = ub.SetDict({'A': 'Ab', 'B': 'Bb', 'C': 'Cb',          })
+        >>> print(a.union(b))
+        >>> print(a.intersection(b))
+        >>> print(a.difference(b))
+        >>> print(a.symmetric_difference(b))
+        {'A': 'Ab', 'B': 'Bb', 'D': 'Da', 'C': 'Cb'}
+        {'A': 'Aa', 'B': 'Ba'}
+        {'D': 'Da'}
+        {'D': 'Da', 'C': 'Cb'}
+        >>> print(a | b)  # union
+        >>> print(a & b)  # intersection
+        >>> print(a - b)  # difference
+        >>> print(a ^ b)  # symmetric_difference
+        {'A': 'Ab', 'B': 'Bb', 'D': 'Da', 'C': 'Cb'}
+        {'A': 'Aa', 'B': 'Ba'}
+        {'D': 'Da'}
+        {'D': 'Da', 'C': 'Cb'}
+
+    Example:
+        >>> import ubelt as ub
+        >>> a = ub.SetDict({'A': 'Aa', 'B': 'Ba',            'D': 'Da'})
+        >>> b = ub.SetDict({'A': 'Ab', 'B': 'Bb', 'C': 'Cb',          })
+        >>> c = ub.SetDict({'A': 'Ac', 'B': 'Bc',                       'E': 'Ec'})
+        >>> d = ub.SetDict({'A': 'Ad',            'C': 'Cd', 'D': 'Dd'})
+        >>> # 3-ary operations
+        >>> print(a.union(b, c))
+        >>> print(a.intersection(b, c))
+        >>> print(a.difference(b, c))
+        >>> print(a.symmetric_difference(b, c))
+        {'A': 'Ac', 'B': 'Bc', 'D': 'Da', 'C': 'Cb', 'E': 'Ec'}
+        {'A': 'Aa', 'B': 'Ba'}
+        {'D': 'Da'}
+        {'D': 'Da', 'C': 'Cb', 'A': 'Ac', 'B': 'Bc', 'E': 'Ec'}
+        >>> # 4-ary operations
+        >>> print(ub.UDict.union(a, b, c, c))
+        >>> print(ub.UDict.intersection(a, b, c, c))
+        >>> print(ub.UDict.difference(a, b, c, d))
+        >>> print(ub.UDict.symmetric_difference(a, b, c, d))
+        {'A': 'Ac', 'B': 'Bc', 'D': 'Da', 'C': 'Cb', 'E': 'Ec'}
+        {'A': 'Aa', 'B': 'Ba'}
+        {}
+        {'B': 'Bc', 'E': 'Ec'}
+
     Example:
         >>> import ubelt as ub
         >>> primes = ub.sdict({v: f'prime_{v}' for v in [2, 3, 5, 7, 11]})
@@ -1229,6 +1344,8 @@ class SetDict(dict):
 
     def __ior__(self, other):
         """
+        The inplace union operator ``|=``.
+
         Example:
             >>> import ubelt as ub
             >>> self = orig_ref = ub.sdict({1: 1, 2: 2, 3: 3})
@@ -1245,6 +1362,8 @@ class SetDict(dict):
 
     def __iand__(self, other):
         """
+        The inplace intersection operator ``&=``.
+
         Example:
             >>> import ubelt as ub
             >>> self = orig_ref = ub.sdict({1: 1, 2: 2, 3: 3})
@@ -1256,13 +1375,15 @@ class SetDict(dict):
             >>> assert self == (orig_val & other)
             self={1: 1, 2: 2}
         """
-        result = self.intersection(other)
-        self.clear()
-        self.update(result)
+        remove_keys = self.keys() - set(other)
+        for k in remove_keys:
+            del self[k]
         return self
 
     def __isub__(self, other):
         """
+        The inplace difference operator ``-=``.
+
         Example:
             >>> import ubelt as ub
             >>> self = orig_ref = ub.sdict({1: 1, 2: 2, 3: 3})
@@ -1293,16 +1414,15 @@ class SetDict(dict):
             >>> assert self is orig_ref
             >>> assert self == (orig_val - other)
         """
-        result = self.difference(other)
-        self.clear()
-        self.update(result)
-        # common = UDict.intersection(self, other)
-        # for k in common:
-        #     self.pop(k, None)
+        remove_keys = self.keys() & set(other)
+        for k in remove_keys:
+            del self[k]
         return self
 
     def __ixor__(self, other):
         """
+        The inplace symmetric difference operator ``^=``.
+
         Example:
             >>> import ubelt as ub
             >>> self = orig_ref = ub.sdict({1: 1, 2: 2, 3: 3})
@@ -1313,19 +1433,24 @@ class SetDict(dict):
             >>> assert self is orig_ref
             >>> assert self == (orig_val ^ other)
         """
-        result = self.symmetric_difference(other)
-        self.clear()
-        self.update(result)
+        other_keys = set(other.keys())
+        remove_keys = self.keys() & other_keys
+        add_keys = other_keys - remove_keys
+        for k in remove_keys:
+            del self[k]
+        for k in add_keys:
+            self[k] = other[k]
         return self
 
     ### Main set operations
 
-    def union(self, *others, cls=None):
+    def union(self, *others, cls=None, merge=None):
         """
         Return the key-wise union of two or more dictionaries.
 
-        For items with intersecting keys, dictionaries towards the end of the
-        sequence are given precedence.
+        Values chosen with *right-most* priority. I.e. for items with
+        intersecting keys, dictionaries towards the end of the sequence are
+        given precedence.
 
         Args:
             self (SetDict | dict):
@@ -1338,8 +1463,16 @@ class SetDict(dict):
             cls (type | None):
                 the desired return dictionary type.
 
+            merge (None | Callable):
+                if specified this function must accept an iterable of values
+                and return a new value to use (which typically is derived from
+                input values). NotImplemented, help wanted.
+
         Returns:
-            dict : whatever the dictionary type of the first argument is
+            dict : items from all input dictionaries. Conflicts are resolved
+                with right-most priority unless ``merge`` is specified.
+                Specific return type is specified by ``cls`` or defaults to the
+                leftmost input.
 
         Example:
             >>> import ubelt as ub
@@ -1355,17 +1488,26 @@ class SetDict(dict):
             >>> print(ub.repr2(res, sort=1, nl=0, si=1))
             {0: B_a, 2: C_c, 3: C_d, 4: B_e, 5: A_f, 7: B_h, 8: C_i, 9: D_j, 10: D_k, 11: D_l}
         """
-        cls = cls or self.__class__
+        if cls is None:
+            # Some subclass-constructors need special handling
+            # Not sure if it is in-scope to do that here or not.
+            # if isinstance(self.__class__, defaultdict):
+            #     ...
+            cls = self.__class__
         args = it.chain([self], others)
-        new = cls(it.chain.from_iterable(d.items() for d in args))
+        if merge is None:
+            new = cls(it.chain.from_iterable(d.items() for d in args))
+        else:
+            raise NotImplementedError('merge function is not yet implemented')
         return new
 
-    def intersection(self, *others, cls=None):
+    def intersection(self, *others, cls=None, merge=None):
         """
         Return the key-wise intersection of two or more dictionaries.
 
-        All items returned will be from the first dictionary for keys that
-        exist in all other dictionaries / sets provided.
+        Values returned with *left-most* priority. I.e. all items returned will
+        be from the first dictionary for keys that exist in all other
+        dictionaries / sets provided.
 
         Args:
             self (SetDict | dict):
@@ -1377,8 +1519,25 @@ class SetDict(dict):
             cls (type | None):
                 the desired return dictionary type.
 
+            merge (None | Callable):
+                if specified this function must accept an iterable of values
+                and return a new value to use (which typically is derived from
+                input values). NotImplemented, help wanted.
+
         Returns:
-            dict : whatever the dictionary type of the first argument is
+            dict : items with keys shared by all the inputs. Values take
+                left-most priority unless ``merge`` is specified. Specific
+                return type is specified by ``cls`` or defaults to the leftmost
+                input.
+
+        Example:
+            >>> import ubelt as ub
+            >>> a = ub.SetDict({'a': 1, 'b': 2, 'd': 4})
+            >>> b = ub.SetDict({'a': 10, 'b': 20, 'c': 30})
+            >>> a.intersection(b)
+            {'a': 1, 'b': 2}
+            >>> a & b
+            {'a': 1, 'b': 2}
 
         Example:
             >>> import ubelt as ub
@@ -1398,17 +1557,20 @@ class SetDict(dict):
         isect_keys = set(self.keys())
         for v in others:
             isect_keys.intersection_update(v)
-        new = cls((k, self[k]) for k in self if k in isect_keys)
+        if merge is None:
+            new = cls((k, self[k]) for k in self if k in isect_keys)
+        else:
+            raise NotImplementedError('merge function is not yet implemented')
         return new
 
-    def difference(self, *others, cls=None):
+    def difference(self, *others, cls=None, merge=None):
         """
         Return the key-wise difference between this dictionary and one or
         more other dictionary / keys.
 
-        The returned items will be from the first dictionary, and will only
-        contain keys that do not appear in any of the other dictionaries /
-        sets.
+        Values returned with *left-most* priority. I.e. the returned items will
+        be from the first dictionary, and will only contain keys that do not
+        appear in any of the other dictionaries / sets.
 
         Args:
             self (SetDict | dict):
@@ -1420,8 +1582,16 @@ class SetDict(dict):
             cls (type | None):
                 the desired return dictionary type.
 
+            merge (None | Callable):
+                if specified this function must accept an iterable of values
+                and return a new value to use (which typically is derived from
+                input values). NotImplemented, help wanted.
+
         Returns:
-            dict : whatever the dictionary type of the first argument is
+            dict : items from the first dictionary with keys not in any of the
+                following inputs. Values take left-most priority unless
+                ``merge`` is specified. Specific return type is specified by
+                ``cls`` or defaults to the leftmost input.
 
         Example:
             >>> import ubelt as ub
@@ -1441,19 +1611,22 @@ class SetDict(dict):
         other_keys = set()
         for v in others:
             other_keys.update(v)
-        # Looping over original keys is important to maintain partial order.
-        new = cls((k, self[k]) for k in self.keys() if k not in other_keys)
+        if merge is None:
+            # Looping over original keys is important to maintain partial order.
+            new = cls((k, self[k]) for k in self.keys() if k not in other_keys)
+        else:
+            raise NotImplementedError('merge function is not yet implemented')
         return new
 
-    def symmetric_difference(self, *others, cls=None):
+    def symmetric_difference(self, *others, cls=None, merge=None):
         """
         Return the key-wise symmetric difference between this dictionary and
         one or more other dictionaries.
 
-        Returns items that are (key-wise) in an odd number of the given
-        dictionaries. This is consistent with the standard n-ary definition of
-        symmetric difference [WikiSymDiff]_ and corresponds with the xor
-        operation.
+        Values chosen with *right-most* priority. Returns items that are
+        (key-wise) in an odd number of the given dictionaries. This is
+        consistent with the standard n-ary definition of symmetric difference
+        [WikiSymDiff]_ and corresponds with the xor operation.
 
         Args:
             self (SetDict | dict):
@@ -1465,8 +1638,16 @@ class SetDict(dict):
             cls (type | None):
                 the desired return dictionary type.
 
+            merge (None | Callable):
+                if specified this function must accept an iterable of values
+                and return a new value to use (which typically is derived from
+                input values). NotImplemented, help wanted.
+
         Returns:
-            dict : whatever the dictionary type of the first argument is
+            dict : items from input dictionaries where the key appears an odd
+                number of times. Values take right-most priority unless
+                ``merge`` is specified. Specific return type is specified by
+                ``cls`` or defaults to the leftmost input.
 
         References:
             .. [WikiSymDiff] https://en.wikipedia.org/wiki/Symmetric_difference
@@ -1488,12 +1669,15 @@ class SetDict(dict):
         """
         cls = cls or self.__class__
         new = cls(self)  # shallow copy
-        for d in others:
-            for k, v in d.items():
-                if k in new:
-                    new.pop(k)
-                else:
-                    new[k] = v
+        if merge is None:
+            for d in others:
+                for k, v in d.items():
+                    if k in new:
+                        new.pop(k)
+                    else:
+                        new[k] = v
+        else:
+            raise NotImplementedError('merge function is not yet implemented')
         return new
 
 
