@@ -34,7 +34,7 @@ def identity(arg=None, *args, **kwargs):
     assigning it to a value.
 
     Args:
-        arg (Any, default=None): The value to return unchanged.
+        arg (Any | None, default=None): The value to return unchanged.
         *args: Ignored
         **kwargs: Ignored
 
@@ -71,7 +71,7 @@ def inject_method(self, func, name=None):
         func (Callable[..., Any]):
             The function to inject (must contain an arg for self).
 
-        name (str, default=None):
+        name (str | None, default=None):
             Name of the method. optional. If not specified the name of the
             function is used.
 
@@ -183,7 +183,17 @@ def compatible(config, func, start=0, keywords=True):
           'a': 2, 'b': 3, 'c': 7,
           'd': 11, 'e': 13, 'f': 17,
         }
-        func(1, 2, **ub.compatible(config, func))
+        funckw = ub.compatible(config, func)
+        func(1, 2, **funckw)
+
+
+        ### While the stdlib inspect.signature is useful, it does not
+        ### have a concise way of getting the subset of the dictionary
+        ### that can be passed as keyword arguments.
+        import inspect
+        sig = inspect.signature(func)
+        funckw2 = ub.udict(config) & sig.parameters
+        ub.udict(report_config) & (sig.parameters)
     """
     import inspect
     sig = inspect.signature(func)
