@@ -61,6 +61,11 @@ class SerialFuture(concurrent.futures.Future):
     """
     Non-threading / multiprocessing version of future for drop in compatibility
     with concurrent.futures.
+
+    Attributes:
+        func (Callable): function to be called
+        args (Tuple): positional arguments to call the function with
+        kw (Dict): keyword arguments to call the function with
     """
     def __init__(self, func, *args, **kw):
         super(SerialFuture, self).__init__()
@@ -145,8 +150,17 @@ class SerialExecutor(object):
         self.max_workers = 0
         return self
 
-    def __exit__(self, ex_type, ex_value, tb):
-        pass
+    def __exit__(self, ex_type, ex_value, ex_traceback):
+        """
+        Args:
+            ex_type (Type[BaseException] | None):
+            ex_value (BaseException | None):
+            ex_traceback (TracebackType | None):
+
+        Returns:
+            bool | None
+        """
+        return False
 
     def submit(self, func, *args, **kw):
         """
@@ -230,8 +244,8 @@ class SerialExecutor(object):
 #             self.loop = asyncio.get_event_loop()
 #     def __enter__(self):
 #         return self
-#     def __exit__(self, ex_type, ex_value, tb):
-#         pass
+#     def __exit__(self, ex_type, ex_value, ex_traceback):
+#         ...
 #     def submit(self, func, *args, **kw):
 #         ...
 #     def shutdown(self):
@@ -255,6 +269,46 @@ class Executor(object):
         * :class:`SerialExecutor`
         * :class:`JobPool`
 
+    In the case where you cant or dont want to use ubelt.Executor you can get
+    similar behavior with the following pure-python snippet:
+
+    .. code:: python
+
+        def Executor(max_workers):
+            # Stdlib-only "ubelt.Executor"-like behavior
+            if max_workers == 1:
+                import contextlib
+                def submit_partial(func, *args, **kwargs):
+                    def wrapper():
+                        return func(*args, **kwargs)
+                    wrapper.result = wrapper
+                    return wrapper
+                executor = contextlib.nullcontext()
+                executor.submit = submit_partial
+            else:
+                from concurrent.futures import ThreadPoolExecutor
+                executor = ThreadPoolExecutor(max_workers=max_workers)
+            return executor
+
+        executor = Executor(0)
+        with executor:
+            jobs = []
+
+            for arg in range(1000):
+                job = executor.submit(chr, arg)
+                jobs.append(job)
+
+            results = []
+            for job in jobs:
+                result = job.result()
+                results.append(result)
+
+        print('results = {}'.format(ub.urepr(results, nl=1)))
+
+
+    Attributes:
+        backend (SerialExecutor | ThreadPoolExecutor | ProcessPoolExecutor):
+
     Example:
         >>> import ubelt as ub
         >>> # Prototype code using simple serial processing
@@ -298,8 +352,18 @@ class Executor(object):
         self.backend.__enter__()
         return self
 
-    def __exit__(self, ex_type, ex_value, tb):
-        return self.backend.__exit__(ex_type, ex_value, tb)
+    def __exit__(self, ex_type, ex_value, ex_traceback):
+        """
+        Args:
+            ex_type (Type[BaseException] | None):
+            ex_value (BaseException | None):
+            ex_traceback (TracebackType | None):
+
+        Returns:
+            bool | None
+        """
+        # Note: the following call will block
+        return self.backend.__exit__(ex_type, ex_value, ex_traceback)
 
     def submit(self, func, *args, **kw):
         """
@@ -358,18 +422,10 @@ class JobPool(object):
     submitted futures for you and 2. providing an as_completed method to
     consume those futures as they are ready.
 
-    Args:
-        mode (str):
-            The backend parallelism mechanism.  Can be either thread, serial,
-            or process. Defaults to 'thread'.
-
-        max_workers (int):
-            number of workers. If 0, serial is forced. Defaults to 0.
-
-        transient (bool):
-            if True, references to jobs will be discarded as they are
-            returned by :func:`as_completed`. Otherwise the ``jobs`` attribute
-            holds a reference to all jobs ever submitted. Default to False.
+    Attributes:
+        executor (Executor): internal executor object
+        jobs (List[Future]): internal job list. Note: do not rely on this attribute, it
+            may change in the future.
 
     Example:
         >>> import ubelt as ub
@@ -385,6 +441,20 @@ class JobPool(object):
         >>> print('final = {!r}'.format(final))
     """
     def __init__(self, mode='thread', max_workers=0, transient=False):
+        """
+        Args:
+            mode (str):
+                The backend parallelism mechanism.  Can be either thread, serial,
+                or process. Defaults to 'thread'.
+
+            max_workers (int):
+                number of workers. If 0, serial is forced. Defaults to 0.
+
+            transient (bool):
+                if True, references to jobs will be discarded as they are
+                returned by :func:`as_completed`. Otherwise the ``jobs`` attribute
+                holds a reference to all jobs ever submitted. Default to False.
+        """
         self.executor = Executor(mode=mode, max_workers=max_workers)
         self.transient = transient
         self.jobs = []
@@ -421,8 +491,17 @@ class JobPool(object):
         self.executor.__enter__()
         return self
 
-    def __exit__(self, a, b, c):
-        self.executor.__exit__(a, b, c)
+    def __exit__(self, ex_type, ex_value, ex_traceback):
+        """
+        Args:
+            ex_type (Type[BaseException] | None):
+            ex_value (BaseException | None):
+            ex_traceback (TracebackType | None):
+
+        Returns:
+            bool | None
+        """
+        return self.executor.__exit__(ex_type, ex_value, ex_traceback)
 
     def _clear_completed(self):
         active_jobs = [job for job in self.jobs if job.running()]
@@ -480,6 +559,9 @@ class JobPool(object):
             self._prog = job_iter
         for job in job_iter:
             if self.transient:
+                # Maybe keep a reference to the job index and then null it out
+                # in our job list? Should probably think about a good
+                # implementation. See kwcoco.CocoDataset._load_multiple
                 self.jobs.remove(job)
             yield job
 
@@ -520,7 +602,15 @@ class JobPool(object):
 
     def __iter__(self):
         """
-        An alternative to as completed
+        An alternative to as completed.
+
+        NOTE:
+            The order of iteration may be changed in the future to be the
+            submission order instead.
+
+        Yields:
+            concurrent.futures.Future:
+                The completed future object containing the results of a job.
 
         Example:
             >>> import ubelt as ub
