@@ -41,7 +41,7 @@ import warnings
 
 __all__ = [
     'Path', 'TempDir', 'augpath', 'shrinkuser', 'userhome', 'ensuredir',
-    'expandpath',
+    'expandpath', 'ChDir',
 ]
 
 
@@ -166,11 +166,11 @@ def userhome(username=None):
 
     Args:
         username (str | None):
-            name of a user on the system. If not specified, the current user is
-            inferred.
+            name of a user on the system. If unspecified, the current user is
+            inferred from standard environment variables.
 
     Returns:
-        str: userhome_dpath - path to the specified home directory
+        str: path to the specified home directory
 
     Raises:
         KeyError: if the specified user does not exist on the system
@@ -179,11 +179,11 @@ def userhome(username=None):
             inferred
 
     Example:
-        >>> from ubelt.util_path import *  # NOQA
         >>> import ubelt as ub
+        >>> import os
         >>> import getpass
         >>> username = getpass.getuser()
-        >>> userhome_target = expanduser('~')
+        >>> userhome_target = os.path.expanduser('~')
         >>> userhome_got1 = ub.userhome()
         >>> userhome_got2 = ub.userhome(username)
         >>> print(f'username={username}')
@@ -236,11 +236,11 @@ def shrinkuser(path, home='~'):
     Args:
         path (str | PathLike): path in system file structure
         home (str): symbol used to replace the home path.
-            Defaults to '~', but you might want to use '$HOME' or
-            '%USERPROFILE%' instead.
+            Defaults to ``'~'``, but you might want to use ``'$HOME'`` or
+            ``'%USERPROFILE%'`` instead.
 
     Returns:
-        str: path - shortened path replacing the home directory with a symbol
+        str: shortened path replacing the home directory with a symbol
 
     Example:
         >>> from ubelt.util_path import *  # NOQA
@@ -270,7 +270,7 @@ def expandpath(path):
         path (str | PathLike): string representation of a path
 
     Returns:
-        str : expanded path
+        str: expanded path
 
     Example:
         >>> from ubelt.util_path import *  # NOQA
@@ -289,41 +289,46 @@ def ensuredir(dpath, mode=0o1777, verbose=0, recreate=False):
     default
 
     Args:
-        dpath (str | PathLike | Tuple[str | PathLike]): dir to ensure. Can also
-            be a tuple to send to join
-        mode (int): octal mode of directory
+        dpath (str | PathLike | Tuple[str | PathLike]):
+            directory to create if it does not exist.
+
+        mode (int):
+            octal permissions if a new directory is created.
+            Defaults to 0o1777.
+
         verbose (int): verbosity
+
         recreate (bool): if True removes the directory and
-            all of its contents and creates a fresh new directory.
-            USE CAREFULLY.
+            all of its contents and creates a new empty directory.
+            DEPRECATED: Use ``ub.Path(dpath).delete().ensuredir()`` instead.
 
     Returns:
-        str: path - the ensured directory
+        str: the ensured directory
 
     SeeAlso:
         :func:`ubelt.Path.ensuredir`
 
-    Note:
-        This function is not thread-safe in Python2
-
     Example:
-        >>> from ubelt.util_path import *  # NOQA
         >>> import ubelt as ub
-        >>> cache_dpath = ub.Path.appdir('ubelt').ensuredir()
-        >>> dpath = join(cache_dpath, 'ensuredir')
-        >>> if exists(dpath):
-        ...     os.rmdir(dpath)
-        >>> assert not exists(dpath)
+        >>> dpath = ub.Path.appdir('ubelt', 'ensuredir')
+        >>> dpath.delete()
+        >>> assert not dpath.exists()
         >>> ub.ensuredir(dpath)
-        >>> assert exists(dpath)
-        >>> os.rmdir(dpath)
+        >>> assert dpath.exists()
+        >>> dpath.delete()
     """
     if isinstance(dpath, (list, tuple)):
         dpath = join(*dpath)
 
     if recreate:
         import ubelt as ub
-        ub.delete(dpath, verbose=verbose)
+        ub.schedule_deprecation(
+            modname='ubelt',
+            migration='Use ``ub.Path(dpath).delete().ensuredir()`` instead', name='recreate',
+            type='argument of ensuredir', deprecate='1.3.0', error='2.0.0',
+            remove='2.1.0',
+        )
+        util_io.delete(dpath, verbose=verbose)
 
     if not exists(dpath):
         if verbose:
@@ -340,11 +345,18 @@ class ChDir:
     Context manager that changes the current working directory and then
     returns you to where you were.
 
+    This is nearly the same as the stdlib :func:`contextlib.chdir`, with the
+    exception that it will do nothing if the input path is None (i.e. the user
+    did not want to change directories).
+
     Args:
         dpath (str | PathLike | None):
             The new directory to work in.
             If None, then the context manager is disabled.
 
+    SeeAlso:
+        :func:`contextlib.chdir`
+
     Example:
         >>> import ubelt as ub
         >>> dpath = ub.Path.appdir('ubelt/tests/chdir').ensuredir()
@@ -352,7 +364,7 @@ class ChDir:
         >>> dir2 = (dpath / 'dir2').ensuredir()
         >>> with ChDir(dpath):
         >>>     assert ub.Path.cwd() == dpath
-        >>>     # changes to the given directory, and then returns back
+        >>>     # change to the given directory, and then returns back
         >>>     with ChDir(dir1):
         >>>         assert ub.Path.cwd() == dir1
         >>>         with ChDir(dir2):
@@ -376,33 +388,50 @@ class ChDir:
         >>>     assert ub.Path.cwd() == dpath
     """
     def __init__(self, dpath):
-        self.context_dpath = dpath
-        self.orig_dpath = None
+        self._context_dpath = dpath
+        self._orig_dpath = None
 
     def __enter__(self):
-        if self.context_dpath is not None:
-            self.orig_dpath = os.getcwd()
-            os.chdir(self.context_dpath)
+        """
+        Returns:
+            ChDir: self
+        """
+        if self._context_dpath is not None:
+            self._orig_dpath = os.getcwd()
+            os.chdir(self._context_dpath)
         return self
 
-    def __exit__(self, a, b, c):
-        if self.context_dpath is not None:
-            os.chdir(self.orig_dpath)
+    def __exit__(self, ex_type, ex_value, ex_traceback):
+        """
+        Args:
+            ex_type (Type[BaseException] | None):
+            ex_value (BaseException | None):
+            ex_traceback (TracebackType | None):
+
+        Returns:
+            bool | None
+        """
+        if self._context_dpath is not None:
+            os.chdir(self._orig_dpath)
 
 
 class TempDir:
     """
     Context for creating and cleaning up temporary directories.
 
-    Note:
-        This class will be DEPRECATED. The exact deprecation version and
-        mitigation plan has not yet been developed.
+    DEPRECATED. Use :mod:`tempfile` instead.
 
     Note:
         This exists because :class:`tempfile.TemporaryDirectory` was
         introduced in Python 3.2. Thus once ubelt no longer supports
         python 2.7, this class will be deprecated.
 
+    Attributes:
+        dpath (str | None)
+
+    Note:
+        # WE MAY WANT TO KEEP THIS FOR WINDOWS.
+
     Example:
         >>> from ubelt.util_path import *  # NOQA
         >>> with TempDir() as self:
@@ -450,7 +479,16 @@ class TempDir:
     def __enter__(self):
         return self.start()
 
-    def __exit__(self, type_, value, trace):
+    def __exit__(self, ex_type, ex_value, ex_traceback):
+        """
+        Args:
+            ex_type (Type[BaseException] | None):
+            ex_value (BaseException | None):
+            ex_traceback (TracebackType | None):
+
+        Returns:
+            bool | None
+        """
         self.cleanup()
 
 
@@ -529,7 +567,6 @@ class Path(_PathBase):
         * :py:data:`pathlib.PurePath.anchor`
         * :py:data:`pathlib.PurePath.name`
         * :py:data:`pathlib.PurePath.parts`
-        * :py:data:`pathlib.PurePath.parts`
         * :py:data:`pathlib.PurePath.parent`
         * :py:data:`pathlib.PurePath.parents`
         * :py:data:`pathlib.PurePath.suffix`
@@ -704,15 +741,14 @@ class Path(_PathBase):
                 Text placed in front of the stem. Defaults to ''.
 
             stemsuffix (str):
-                Text placed between the stem and extension. Default to ''.
-                Note: this is just called suffix in :func:`ub.augpath`.
+                Text placed between the stem and extension. Defaults to ''.
 
             ext (str | None):
                 If specified, replaces the extension
 
             stem (str | None):
                 If specified, replaces the stem (i.e. basename without
-                extension). Note: named base in :func:`augpath`.
+                extension).
 
             dpath (str | PathLike | None):
                 If specified, replaces the specified "relative" directory,
@@ -749,23 +785,18 @@ class Path(_PathBase):
 
             .. code:: python
 
-            p = ub.Path('the.entire.fname.or.dname.is.the.name.exe')
-            print(f'p     ={p}')
-            print(f'p.name={p.name}')
-            p = ub.Path('the.stem.ends.here.ext')
-            print(f'p     ={p}')
-            print(f'p.stem={p.stem}')
-            p = ub.Path('only.the.last.dot.is.the.suffix')
-            print(f'p       ={p}')
-            print(f'p.suffix={p.suffix}')
-            p = ub.Path('but.all.suffixes.can.be.recovered')
-            print(f'p         ={p}')
-            print(f'p.suffixes={p.suffixes}')
-
-            p.name
-            p.stem
-            p.suffixes
-            p.parts
+                p = ub.Path('the.entire.fname.or.dname.is.the.name.exe')
+                print(f'p     ={p}')
+                print(f'p.name={p.name}')
+                p = ub.Path('the.stem.ends.here.ext')
+                print(f'p     ={p}')
+                print(f'p.stem={p.stem}')
+                p = ub.Path('only.the.last.dot.is.the.suffix')
+                print(f'p       ={p}')
+                print(f'p.suffix={p.suffix}')
+                p = ub.Path('but.all.suffixes.can.be.recovered')
+                print(f'p         ={p}')
+                print(f'p.suffixes={p.suffixes}')
 
         Example:
             >>> import ubelt as ub
@@ -823,7 +854,6 @@ class Path(_PathBase):
             )
             if not stemsuffix:
                 stemsuffix = suffix
-            import warnings
             warnings.warn(
                 'DEVELOPER NOTICE: The ubelt.Path.augment function may '
                 'experience a BACKWARDS INCOMPATIBLE update in the future '
@@ -878,20 +908,24 @@ class Path(_PathBase):
         """
         Concise alias of ``self.mkdir(parents=True, exist_ok=True)``
 
+        Args:
+            mode (int):
+                octal permissions if a new directory is created.
+                Defaults to 0o777.
+
         Returns:
             Path: returns itself
 
         Example:
             >>> import ubelt as ub
             >>> cache_dpath = ub.Path.appdir('ubelt').ensuredir()
-            >>> dpath = ub.Path(join(cache_dpath, 'ensuredir'))
-            >>> if dpath.exists():
-            ...     os.rmdir(dpath)
+            >>> dpath = ub.Path(cache_dpath, 'newdir')
+            >>> dpath.delete()
             >>> assert not dpath.exists()
             >>> dpath.ensuredir()
             >>> assert dpath.exists()
             >>> dpath.rmdir()
-            """
+        """
         self.mkdir(mode=mode, parents=True, exist_ok=True)
         return self
 
@@ -900,10 +934,11 @@ class Path(_PathBase):
         Create a new directory at this given path.
 
         Note:
-            The ubelt variant is the same, except it returns the path as well.
+            The ubelt extension is the same as the original pathlib method,
+            except this returns returns the path instead of None.
 
         Args:
-            mode (int) : perms
+            mode (int) : permission bits
             parents (bool) : create parents
             exist_ok (bool) : fail if exists
 
@@ -924,14 +959,11 @@ class Path(_PathBase):
 
         Example:
             >>> import ubelt as ub
-            >>> #home_v1 = ub.Path('$HOME').expand()
-            >>> home_v2 = ub.Path('~/').expand()
-            >>> assert isinstance(home_v2, ub.Path)
-            >>> home_v3 = ub.Path.home()
-            >>> #print('home_v1 = {!r}'.format(home_v1))
+            >>> home_v1 = ub.Path('~/').expand()
+            >>> home_v2 = ub.Path.home()
+            >>> print('home_v1 = {!r}'.format(home_v1))
             >>> print('home_v2 = {!r}'.format(home_v2))
-            >>> print('home_v3 = {!r}'.format(home_v3))
-            >>> assert home_v3 == home_v2 # == home_v1
+            >>> assert home_v1 == home_v2
         """
         return self.expandvars().expanduser()
 
@@ -953,8 +985,8 @@ class Path(_PathBase):
         """
         A convenience function to list all paths in a directory.
 
-        This is simply a wraper around iterdir that returns the results as a
-        list instead of a generator. This is mainly for faster navigation in
+        This is a wrapper around iterdir that returns the results as a list
+        instead of a generator. This is mainly for faster navigation in
         IPython. In production code ``iterdir`` or ``glob`` should be used
         instead.
 
@@ -963,7 +995,7 @@ class Path(_PathBase):
                 if specified, performs a glob instead of an iterdir.
 
         Returns:
-            List[Path]: an eagerly evaluated list of paths
+            List['Path']: an eagerly evaluated list of paths
 
         Note:
             When pattern is specified only paths matching the pattern are
@@ -1002,9 +1034,18 @@ class Path(_PathBase):
         else:
             return list(self.glob(pattern))
 
+    # TODO:
+    # def _glob(self):
+    #     """
+    #     I would like some way of globbing using patterns contained in the path
+    #     itself. Perhaps this goes into expand?
+    #     """
+    #     import glob
+    #     yield from map(self.__class__, glob.glob(self))
+
     def shrinkuser(self, home='~'):
         """
-        Shrinks your home dir by replacing it with a tilde.
+        Shrinks your home directory by replacing it with a tilde.
 
         This is the inverse of :func:`os.path.expanduser`.
 
@@ -1014,7 +1055,7 @@ class Path(_PathBase):
                 '%USERPROFILE%' instead.
 
         Returns:
-            Path: path - shortened path replacing the home directory with a symbol
+            Path: shortened path replacing the home directory with a symbol
 
         Example:
             >>> import ubelt as ub
@@ -1153,7 +1194,7 @@ class Path(_PathBase):
 
     def endswith(self, suffix, *args):
         """
-        Test if the fspath representation endswith a particular string
+        Test if the fspath representation ends with ``suffix``.
 
         Allows ubelt.Path to be a better drop-in replacement when working with
         string-based paths.
@@ -1167,7 +1208,7 @@ class Path(_PathBase):
                 end (int): if specified stop testing at this position.
 
         Returns:
-            bool: True if any of the suffixes are matched.
+            bool: True if any of the suffixes match.
 
         Example:
             >>> import ubelt as ub
@@ -1188,7 +1229,7 @@ class Path(_PathBase):
 
     def startswith(self, prefix, *args):
         """
-        Test if the fspath representation startswith a particular string
+        Test if the fspath representation starts with ``prefix``.
 
         Allows ubelt.Path to be a better drop-in replacement when working with
         string-based paths.
@@ -1202,7 +1243,7 @@ class Path(_PathBase):
                 end (int): if specified stop testing at this position.
 
         Returns:
-            bool: True if any of the prefixes are matched.
+            bool: True if any of the prefixes match.
 
         Example:
             >>> import ubelt as ub
@@ -1248,7 +1289,6 @@ class Path(_PathBase):
 
         By default files are never overwritten and symlinks are copied as-is.
 
-
         At a basic level (i.e. ignoring symlinks) for each path argument
         (``src`` and ``dst``) these can either be files, directories, or not
         exist. Given these three states, the following table summarizes how
@@ -1256,17 +1296,17 @@ class Path(_PathBase):
 
         TextArt:
 
-            +----------+------------------------+------------------------+------------+
-            | dst      | dir                    | file                   | no-exist   |
-            +----------+                        |                        |            |
-            | src      |                        |                        |            |
-            +==========+========================+========================+============+
-            | dir      | error-or-overwrite-dst | error                  | dst        |
-            +----------+------------------------+------------------------+------------+
-            | file     | dst / src.name         | error-or-overwrite-dst | dst        |
-            +----------+------------------------+------------------------+------------+
-            | no-exist | error                  | error                  | error      |
-            +----------+------------------------+------------------------+------------+
+            +----------+------------------------+------------------------+----------+
+            | dst      | dir                    | file                   | no-exist |
+            +----------+                        |                        |          |
+            | src      |                        |                        |          |
+            +==========+========================+========================+==========+
+            | dir      | error-or-overwrite-dst | error                  | dst      |
+            +----------+------------------------+------------------------+----------+
+            | file     | dst / src.name         | error-or-overwrite-dst | dst      |
+            +----------+------------------------+------------------------+----------+
+            | no-exist | error                  | error                  | error    |
+            +----------+------------------------+------------------------+----------+
 
         In general, the contents of src will be the contents of dst, except for
         the one case where a file is copied into an existing directory. In this
@@ -1290,18 +1330,18 @@ class Path(_PathBase):
             import pandas as pd
             df = pd.DataFrame(rows)
             piv = df.pivot(['src'], ['dst'], 'result')
-            print(piv.to_markdown(tablefmt="presto"))
-            print(piv.to_markdown(tablefmt="pretty", index=True))
             print(piv.to_markdown(tablefmt="grid", index=True))
 
+            See: ~/code/ubelt/tests/test_path.py for test cases
+
         Args:
             dst (str | PathLike):
-                if `src` is a file and `dst` does not exist, copies this to `dst`
-                if `src` is a file and `dst` is a directory, copies this to `dst / src.name`
+                if ``src`` is a file and ``dst`` does not exist, copies this to ``dst``
+                if ``src`` is a file and ``dst`` is a directory, copies this to ``dst / src.name``
 
-                if `src` is a directory and `dst` does not exist, copies this to `dst`
-                if `src` is a directory and `dst` is a directory, errors unless
-                overwrite is True, in which case, copies this to `dst` and
+                if ``src`` is a directory and ``dst`` does not exist, copies this to ``dst``
+                if ``src`` is a directory and ``dst`` is a directory, errors unless
+                overwrite is True, in which case, copies this to ``dst`` and
                 overwrites anything conflicting path.
 
             follow_file_symlinks (bool):
@@ -1327,13 +1367,22 @@ class Path(_PathBase):
                 otherwise the file will be overwritten.
 
         Returns:
-            Path: where the path was actually copied to
+            Path: where the path was copied to
 
         Note:
             This is implemented with a combination of :func:`shutil.copy`,
-            :func:`shutil.copy2`, and :func:`shutil.copytree`, but the The
-            defaults and behavior here are noticably different (and hopefully
-            safer and more intuitive).
+            :func:`shutil.copy2`, and :func:`shutil.copytree`, but the defaults
+            and behavior here are different (and ideally safer and more
+            intuitive).
+
+        Note:
+            Unlike cp on Linux, copying a src directory into a dst directory
+            will not implicitly add the src directory name to the dst
+            directory. This means we cannot copy directory ``<parent>/<dname>``
+            to ``<dst>`` and expect the result to be ``<dst>/<dname>``.
+
+            Conceptually you can expect ``<parent>/<dname>/<contents>``
+            to exist in ``<dst>/<contents>``.
 
         Example:
             >>> import ubelt as ub
@@ -1356,15 +1405,7 @@ class Path(_PathBase):
             >>> paths['empty_dpath'].copy((clone1 / 'empty_dpath_alt').ensuredir(), overwrite=True)
             >>> paths['nested_dpath'].copy(clone0 / 'nested_dpath')
             >>> paths['nested_dpath'].copy((clone1 / 'nested_dpath_alt').ensuredir(), overwrite=True)
-
-        Ignore:
-            import xdev
-            xdev.tree_repr(dpath)
-            xdev.tree_repr(clone0, max_files=10)
-            xdev.tree_repr(clone1, max_files=10)
         """
-        warnings.warn('The ub.Path.copy function is experimental and may change, '
-                      'in corner cases. Primary cases seem stable.')
         import shutil
         copy_function = self._request_copy_function(
             follow_file_symlinks=follow_file_symlinks,
@@ -1388,7 +1429,7 @@ class Path(_PathBase):
                     raise FileExistsError('Cannot overwrite existing file unless overwrite=True')
             dst = copy_function(self, dst)
         else:
-            raise Exception
+            raise FileExistsError('The source path does not exist')
         return Path(dst)
 
     def move(self, dst, follow_file_symlinks=False, follow_dir_symlinks=False,
@@ -1428,14 +1469,14 @@ class Path(_PathBase):
 
             This is implemented via :func:`shutil.move`, which depends heavily
             on :func:`os.rename` semantics. For this reason, this function
-            error if it would overwrite any data. If you want an overwriting
-            variant of move we recommend you either either copy the data, and
-            then delete the original (potentially inefficient), or use
-            :func:`shutil.move` directly if you know how :func:`os.rename`
+            will error if it would overwrite any data. If you want an
+            overwriting variant of move we recommend you either either copy the
+            data, and then delete the original (potentially inefficient), or
+            use :func:`shutil.move` directly if you know how :func:`os.rename`
             works on your system.
 
         Returns:
-            Path: where the path was actually moved to
+            Path: where the path was moved to
 
         Example:
             >>> import ubelt as ub
@@ -1448,28 +1489,12 @@ class Path(_PathBase):
             >>> paths['dpath01'] = (dpath / 'dpath0' / 'sub1').ensuredir()
             >>> print('paths = {}'.format(ub.repr2(paths, nl=1)))
             >>> assert all(p.exists() for p in paths.values())
-            >>> # xdev.tree_repr(dpath, max_files=10)
             >>> paths['dpath0'].move(dpath / 'dpath1')
-            >>> # xdev.tree_repr(dpath, max_files=10)
-
-        Ignore:
-            import xdev
-            xdev.tree_repr(dpath)
         """
-        warnings.warn('The ub.Path.move function is experimental and may change! '
-                      'Do not rely on this behavior yet!')
-
         # Behave more like POSIX move to avoid potential confusing behavior
         if exists(dst):
             raise FileExistsError(
                 'Moves are only allowed to locations that dont exist')
-        # if os.path.isdir(dst):
-        #     with os.scandir(dst) as itr:
-        #         is_empty = next(itr, None) is None
-        #     if not is_empty:
-        #         raise IOError(f'Cannot move {self!r} to {dst!r}. '
-        #                       'Directory not empty')
-
         import shutil
         copy_function = self._request_copy_function(
             follow_file_symlinks=follow_file_symlinks,
@@ -1487,7 +1512,6 @@ if sys.version_info[0:2] < (3, 8):  # nocover
         A vendored shutil.copytree for older pythons based on the 3.10
         implementation
         """
-        # import stat
         from shutil import Error, copystat, copy2, copy
         with os.scandir(src) as itr:
             entries = list(itr)
