@@ -1,5 +1,6 @@
 """
-Functions for capturing and redirecting IO streams.
+Functions for capturing and redirecting IO streams with optional
+tee-functionality.
 
 The :class:`CaptureStdout` captures all text sent to stdout and optionally
 prevents it from actually reaching stdout.
@@ -23,14 +24,22 @@ class TeeStringIO(io.StringIO):
     An IO object that writes to itself and another IO stream.
 
     Attributes:
-        redirect (io.IOBase): The other stream to write to.
+        redirect (io.IOBase | None): The other stream to write to.
 
     Example:
         >>> import ubelt as ub
+        >>> import io
         >>> redirect = io.StringIO()
         >>> self = ub.TeeStringIO(redirect)
+        >>> self.write('spam')
+        >>> assert self.getvalue() == 'spam'
+        >>> assert redirect.getvalue() == 'spam'
     """
     def __init__(self, redirect=None):
+        """
+        Args:
+            redirect (io.IOBase): The other stream to write to.
+        """
         self.redirect = redirect  # type: io.IOBase
         super().__init__()
 
@@ -43,6 +52,8 @@ class TeeStringIO(io.StringIO):
             self.buffer = redirect.buffer  # Py3.
         else:
             self.buffer = redirect
+        # Note: mypy doesn't like this type
+        # buffer (io.BufferedIOBase | io.IOBase | None): the redirected buffer attribute
 
     def isatty(self):  # nocover
         """
@@ -51,6 +62,12 @@ class TeeStringIO(io.StringIO):
         Note:
             Needed for ``IPython.embed`` to work properly when this class is
             used to override stdout / stderr.
+
+        SeeAlso:
+            :meth:`io.IOBase.isatty`
+
+        Returns:
+            bool
         """
         return (self.redirect is not None and
                 hasattr(self.redirect, 'isatty') and self.redirect.isatty())
@@ -60,14 +77,33 @@ class TeeStringIO(io.StringIO):
         Returns underlying file descriptor of the redirected IOBase object
         if one exists.
 
+        Returns:
+            int : the integer corresponding to the file descriptor
+
+        SeeAlso:
+            :meth:`io.IOBase.fileno`
+
         Example:
+            >>> import ubelt as ub
+            >>> dpath = ub.Path.appdir('ubelt/tests/util_stream').ensuredir()
+            >>> fpath = dpath / 'fileno-test.txt'
+            >>> with open(fpath, 'w') as file:
+            >>>     self = ub.TeeStringIO(file)
+            >>>     descriptor = self.fileno()
+            >>>     print(f'descriptor={descriptor}')
+            >>>     assert isinstance(descriptor, int)
+
+        Example:
+            >>> # Test errors
             >>> # Not sure the best way to test, this func is important for
             >>> # capturing stdout when ipython embedding
+            >>> import io
             >>> import pytest
+            >>> import ubelt as ub
             >>> with pytest.raises(io.UnsupportedOperation):
-            >>>     TeeStringIO(redirect=io.StringIO()).fileno()
+            >>>     ub.TeeStringIO(redirect=io.StringIO()).fileno()
             >>> with pytest.raises(io.UnsupportedOperation):
-            >>>     TeeStringIO(None).fileno()
+            >>>     ub.TeeStringIO(None).fileno()
         """
         if self.redirect is not None:
             return self.redirect.fileno()
@@ -79,6 +115,15 @@ class TeeStringIO(io.StringIO):
         """
         Gets the encoding of the `redirect` IO object
 
+        FIXME:
+            My complains that this violates the Liskov substitution principle
+            because the return type can be str or None, whereas the parent
+            class always returns a None. In the future we may raise an exception
+            instead of returning None.
+
+        SeeAlso:
+            :py:obj:`io.TextIOBase.encoding`
+
         Example:
             >>> import ubelt as ub
             >>> redirect = io.StringIO()
@@ -88,14 +133,40 @@ class TeeStringIO(io.StringIO):
             >>> redirect = io.TextIOWrapper(io.StringIO())
             >>> assert ub.TeeStringIO(redirect).encoding is redirect.encoding
         """
+        # mypy correctly complains if we include the return type, but we need
+        # to keep this buggy behavior for legacy reasons.
+        # Returns:
+        #     None | str
         if self.redirect is not None:
             return self.redirect.encoding
         else:
             return super().encoding
 
+    @encoding.setter
+    def encoding(self, value):
+        # Adding a setter to make mypy happy
+        raise Exception('Cannot set encoding attribute')
+
     def write(self, msg):
         """
         Write to this and the redirected stream
+
+        Args:
+            msg (str): the data to write
+
+        SeeAlso:
+            :meth:`io.TextIOBase.write`
+
+        Example:
+            >>> import ubelt as ub
+            >>> dpath = ub.Path.appdir('ubelt/tests/util_stream').ensuredir()
+            >>> fpath = dpath / 'write-test.txt'
+            >>> with open(fpath, 'w') as file:
+            >>>     self = ub.TeeStringIO(file)
+            >>>     n = self.write('hello world')
+            >>>     assert n == 11
+            >>> assert self.getvalue() == 'hello world'
+            >>> assert fpath.read_text() == 'hello world'
         """
         if self.redirect is not None:
             self.redirect.write(msg)
@@ -104,6 +175,9 @@ class TeeStringIO(io.StringIO):
     def flush(self):  # nocover
         """
         Flush to this and the redirected stream
+
+        SeeAlso:
+            :meth:`io.IOBase.flush`
         """
         if self.redirect is not None:
             self.redirect.flush()
@@ -118,18 +192,24 @@ class CaptureStream(object):
 
 class CaptureStdout(CaptureStream):
     r"""
-    Context manager that captures stdout and stores it in an internal stream
+    Context manager that captures stdout and stores it in an internal stream.
 
-    Args:
-        suppress (bool, default=True):
-            if True, stdout is not printed while captured
-        enabled (bool, default=True):
-            does nothing if this is False
-        **kwargs : used for backwards compatibility with misspelled
-            deprecated params.
+    Depending on the value of ``supress``, the user can control if stdout is
+    printed (i.e. if stdout is tee-ed or supressed) while it is being captured.
 
     SeeAlso:
-        :func:`contextlib.redirect_stdout`
+        :func:`contextlib.redirect_stdout` - similar, but does not have the
+            ability to print stdout while it is being captured.
+
+    Attributes:
+        text (str | None): internal storage for the most recent part
+
+        parts (List[str]): internal storage for all parts
+
+        cap_stdout (None | TeeStringIO): internal stream proxy
+
+        orig_stdout (io.TextIOBase):
+            internal pointer to the original stdout stream
 
     Example:
         >>> import ubelt as ub
@@ -157,6 +237,15 @@ class CaptureStdout(CaptureStream):
         >>> assert self.text is None
     """
     def __init__(self, suppress=True, enabled=True):
+        """
+        Args:
+            suppress (bool):
+                if True, stdout is not printed while captured.
+                Defaults to True.
+
+            enabled (bool):
+                does nothing if this is False. Defaults to True.
+        """
         self.text = None
         self._pos = 0  # keep track of how much has been logged
         self.parts = []
@@ -211,13 +300,20 @@ class CaptureStdout(CaptureStream):
         self.cap_stdout.close()
         self.cap_stdout = None
 
-    def __exit__(self, type_, value, trace):
+    def __exit__(self, ex_type, ex_value, ex_traceback):
+        """
+        Args:
+            ex_type (Type[BaseException] | None):
+            ex_value (BaseException | None):
+            ex_traceback (TracebackType | None):
+
+        Returns:
+            bool | None
+        """
         if self.enabled:
             try:
                 self.log_part()
-            except Exception:  # nocover
-                raise
             finally:
                 self.stop()
-        if trace is not None:
+        if ex_traceback is not None:
             return False  # return a falsey value on error
